package com.jpattern.orm.generator.javassist;

import com.jpattern.javassist.ClassPool;
import com.jpattern.javassist.LoaderClassPath;
import com.jpattern.orm.classmapper.IClassMapper;
import com.jpattern.orm.generator.IOrmPersistor;
import com.jpattern.orm.generator.IPersistorGenerator;

/**
 * 
 * @author cinafr
 *
 * @param <T>
 */
public class JavassistPersistorGenerator<T> implements IPersistorGenerator<T> {

	private static String BASE_PACKAGE_NAME = JavassistPersistorGenerator.class.getPackage() + ".generated.";
	private static Boolean CLASS_POOL_INITIALISED = Boolean.valueOf(false);

	private synchronized static ClassPool getClassPool() {
		synchronized (CLASS_POOL_INITIALISED) {
			final ClassPool pool = ClassPool.getDefault();
			if (!CLASS_POOL_INITIALISED) {
				pool.insertClassPath(new LoaderClassPath(Thread.currentThread().getContextClassLoader()));
				CLASS_POOL_INITIALISED = true;
			}
			return pool;
		}
	}

	private final IClassMapper<T> classMapper;
	private final String classNameSuffix;

	public JavassistPersistorGenerator(IClassMapper<T> classMapper, String classNameSuffix) {
		this.classMapper = classMapper;
		this.classNameSuffix = classNameSuffix;
	}

	@Override
	public IOrmPersistor<T> generate() throws Exception {
		final String className = BASE_PACKAGE_NAME + this.classMapper.getMappedClass().getCanonicalName() + this.classNameSuffix;

		final Class<IOrmPersistor<T>> clazz = new JavassistPersistorDump<T>(getClassPool(), className, this.classMapper).createClass();

		return clazz.newInstance();
	}

}